@@ -12,12 +12,18 @@ License: GPLv2
 #include "globals.h"
 #include "defs.h"
 
+#define FOOLSM_CONFIG_FILE ETCDIR "/foolsm.conf"
+
 static char *prog = NULL;
 static int cont = TRUE;
 static int dump = FALSE;
 static int ident = 0;
 static int reload_cfg = FALSE;
 static int dump_if_list = FALSE;
+static char *configfile = FOOLSM_CONFIG_FILE;
+static char *pidfile = "/var/run/foolsm.pid";
+static int nodaemon = 0;
+static char *status_str[] = { "down", "up", "unknown", "long_down" };
 
 void set_prog(char *val)
 {
@@ -84,4 +90,39 @@ int get_dump_if_list(void)
 	return(dump_if_list);
 }
 
+void set_configfile(char *val)
+{
+	configfile = val;
+}
+
+char *get_configfile(void)
+{
+	return(configfile);
+}
+
+void set_pidfile(char *val)
+{
+	pidfile = val;
+}
+
+char *get_pidfile(void)
+{
+	return(pidfile);
+}
+
+void set_nodaemon(const int val)
+{
+	nodaemon = val;
+}
+
+int get_nodaemon(void)
+{
+	return(nodaemon);
+}
+
+char *get_status_str(STATUS val)
+{
+	return(status_str[val]);
+}
+
 /* EOF */
