@@ -12,11 +12,15 @@ License: GPLv2
 #define __PLUGIN_EXPORT_H__
 
 #include "config.h"
-#include "lsm.h"
+#include "foolsm.h"
 
 void plugin_export_init(void);
 void plugin_export(CONFIG *first);
 
+#ifndef NO_PLUGIN_EXPORT_STATUS
+void plugin_export_status(CONFIG *first);
+#endif
+
 #endif
 
 #endif
